@@ -1,18 +1,21 @@
 import logging
+from warnings import warn
 
+# import altair as alt
 import matplotlib.patches as patches
 import matplotlib.pyplot as plt
 import networkx as nx
 import numpy as np
-from more_itertools import unique_everseen
 from matplotlib.cm import get_cmap
+from matplotlib.lines import Line2D
 from matplotlib.path import Path
+from more_itertools import unique_everseen
 
 from .geometry import (
     circos_radius,
     get_cartesian,
-    node_theta,
     group_theta,
+    node_theta,
     text_alignment,
 )
 from .polcart import to_degrees
@@ -20,9 +23,11 @@ from .utils import (
     cmaps,
     infer_data_type,
     is_data_diverging,
-    num_discrete_groups,
-    n_group_colorpallet,
     items_in_groups,
+    n_group_colorpallet,
+    num_discrete_groups,
+    to_pandas_edges,
+    to_pandas_nodes,
 )
 
 
@@ -87,6 +92,12 @@ class BasePlot(object):
 
     :param edgeprops: A `matplotlib-compatible `props` dictioanry.
     :type edgeprops: `dict`
+
+    :param fontsize: A text property for editing size of labels in graph
+    :type fontsize: int
+
+    :param fontfamily: A text property to define font family. Valid inputs ('serif', 'sans-serif', 'fantasy', 'monospace').
+    :type fontfamily: string
     """  # noqa
 
     def __init__(
@@ -106,7 +117,9 @@ class BasePlot(object):
         node_label_color=False,
         group_label_position=None,
         group_label_color=False,
-        **kwargs
+        fontsize=10,
+        fontfamily="serif",
+        **kwargs,
     ):
         super(BasePlot, self).__init__()
         # Set graph object
@@ -121,6 +134,11 @@ class BasePlot(object):
 
         # Set node radius
         self.node_size = node_size
+        if self.node_size:
+            self.node_sizes = []
+            self.compute_node_sizes()
+        else:
+            self.node_sizes = [1] * len(self.nodes)
 
         # Set node colors
         self.node_color = node_color
@@ -137,6 +155,12 @@ class BasePlot(object):
 
         # Set edge properties
         self.edge_width = edge_width
+        if self.edge_width:
+            self.edge_widths = []
+            self.compute_edge_widths()
+        else:
+            self.edge_widths = [1] * len(self.edges)
+
         self.edge_color = edge_color
         if self.edge_color:
             self.edge_colors = []
@@ -191,6 +215,13 @@ class BasePlot(object):
             else:
                 self.group_label_color = ["black"] * len(self.nodes)
 
+        # set text properties
+        valid_fonts = ["serif", "sans-serif", "fantasy", "monospace"]
+        if fontfamily not in valid_fonts:
+            raise ValueError(f"fontfamily should be one of {valid_fonts}")
+        self.fontfamily = fontfamily
+        self.fontsize = fontsize
+
     def check_data_types(self, data_types):
         """
         Checks the data_types passed into the Plot constructor and makes sure
@@ -229,7 +260,7 @@ class BasePlot(object):
 
     def compute_node_colors(self):
         """Compute the node colors. Also computes the colorbar."""
-        data = [self.graph.node[n][self.node_color] for n in self.nodes]
+        data = [self.graph.nodes[n][self.node_color] for n in self.nodes]
 
         if self.group_order == "alphabetically":
             data_reduced = sorted(list(set(data)))
@@ -308,6 +339,22 @@ class BasePlot(object):
             )
             self.sm._A = []
 
+    def compute_node_sizes(self):
+        """Compute the node sizes."""
+        if type(self.node_size) is str:
+            nodes = self.graph.nodes
+            self.node_sizes = [nodes[n][self.node_size] for n in self.nodes]
+        else:
+            self.node_sizes = self.node_size
+
+    def compute_edge_widths(self):
+        """Compute the edge widths."""
+        if type(self.edge_width) is str:
+            edges = self.graph.edges
+            self.edge_widths = [edges[n][self.edge_width] for n in self.edges]
+        else:
+            self.edge_widths = self.edge_width
+
     def compute_group_label_positions(self):
         """Computes the position of each group label according to the wanted
         position ("beginning", "middle", "end")
@@ -442,20 +489,31 @@ class CircosPlot(BasePlot):
     :param node_label_layout: which/whether (a) node layout is used,
         either 'rotation', 'numbers' or None
     :type node_label_layout: `string`
+    :param group_label_offset: how much to offset the group labels, so that
+        they are not overlapping with node labels.
+    :type group_label_offset: `float` or `int`
     """
 
     def __init__(self, graph, **kwargs):
         """Create the CircosPlot.
         """
 
+        # A CircosPlot only makes sense for at least 3 nodes
+        assert len(graph.nodes) >= 3
         # Node labels are specified in the node_label_layout argument
         specified_layout = kwargs.pop("node_label_layout", None)
         # Verify that the provided input is legitimate
         valid_node_label_layouts = (None, "rotation", "numbers")
         assert specified_layout in valid_node_label_layouts
-        # Store the noda label layout
+        # Store the node label layout
         self.node_label_layout = specified_layout
 
+        # Group labels' radius can be offset by a certain amount
+        group_label_offset = kwargs.pop("group_label_offset", 0)
+        assert group_label_offset >= 0
+        # Store the group label offset
+        self.group_label_offset = group_label_offset
+
         #
         super(CircosPlot, self).__init__(graph, **kwargs)
 
@@ -464,10 +522,13 @@ class CircosPlot(BasePlot):
         Computes the x,y positions of the group labels.
         """
         assert self.group_label_position in ["beginning", "middle", "end"]
-        data = [self.graph.node[n][self.node_grouping] for n in self.nodes]
+        data = [self.graph.nodes[n][self.node_grouping] for n in self.nodes]
         node_length = len(data)
         groups = items_in_groups(data)
-        radius = 1.02 * (self.plot_radius + self.nodeprops["radius"])
+
+        edge_of_plot = self.plot_radius + self.nodeprops["radius"]
+        # The 1.02 serves as padding
+        radius = 1.02 * edge_of_plot + self.group_label_offset
         xs = []
         ys = []
         has = []
@@ -688,7 +749,8 @@ class CircosPlot(BasePlot):
                         rotation=rot,
                         rotation_mode="anchor",
                         color=self.node_label_color[i],
-                        fontsize=10,
+                        fontsize=self.fontsize,
+                        family=self.fontfamily,
                     )
 
                 # ----- Node label numbering layout -----
@@ -704,6 +766,8 @@ class CircosPlot(BasePlot):
                         ha=label_ha,
                         va=label_va,
                         color=self.node_label_color[i],
+                        fontsize=self.fontsize,
+                        family=self.fontfamily,
                     )
 
                     # Add numbers to nodes
@@ -722,6 +786,8 @@ class CircosPlot(BasePlot):
                         ha=label_ha,
                         va=label_va,
                         color=self.node_label_color[i],
+                        fontsize=self.fontsize,
+                        family=self.fontfamily,
                     )
 
     def draw_edges(self):
@@ -738,9 +804,10 @@ class CircosPlot(BasePlot):
             ]
             color = self.edge_colors[i]
             codes = [Path.MOVETO, Path.CURVE3, Path.CURVE3]
+            lw = self.edge_widths[i]
             path = Path(verts, codes)
             patch = patches.PathPatch(
-                path, lw=1, edgecolor=color, zorder=1, **self.edgeprops
+                path, lw=lw, edgecolor=color, zorder=1, **self.edgeprops
             )
             self.ax.add_patch(patch)
 
@@ -761,6 +828,8 @@ class CircosPlot(BasePlot):
                 ha=label_ha,
                 va=label_va,
                 color=color,
+                fontsize=self.fontsize,
+                family=self.fontfamily,
             )
 
 
@@ -971,12 +1040,12 @@ class ArcPlot(BasePlot):
         assumed to be equal to 0.5 units. Nodes are placed at integer
         locations.
         """
-        xs = []
-        ys = []
-
-        for node in self.nodes:
-            xs.append(self.nodes.index(node))
-            ys.append(0)
+        xs = [0] * len(self.nodes)
+        ys = [max(self.node_sizes)] * len(self.nodes)
+        for i, _ in enumerate(self.nodes[1:], start=1):
+            prev_r = self.node_sizes[i - 1] / 2
+            curr_r = self.node_sizes[i] / 2
+            xs[i] = xs[i - 1] + prev_r + curr_r
 
         self.node_coords = {"x": xs, "y": ys}
 
@@ -984,13 +1053,13 @@ class ArcPlot(BasePlot):
         """
         Draw nodes to screen.
         """
-        node_r = 1
+        node_r = self.node_sizes
         for i, node in enumerate(self.nodes):
             x = self.node_coords["x"][i]
             y = self.node_coords["y"][i]
             color = self.node_colors[i]
             node_patch = patches.Ellipse(
-                (x, y), node_r, node_r, lw=0, color=color, zorder=2
+                (x, y), node_r[i], node_r[i], lw=0, color=color, zorder=2
             )
             self.ax.add_patch(node_patch)
 
@@ -1015,14 +1084,148 @@ class ArcPlot(BasePlot):
 
             verts = [(start_x, start_y), (middle_x, middle_y), (end_x, end_y)]
 
+            color = self.edge_colors[i]
             codes = [Path.MOVETO, Path.CURVE3, Path.CURVE3]
-
+            lw = self.edge_widths[i]
             path = Path(verts, codes)
-            patch = patches.PathPatch(path, lw=1, zorder=1, **self.edgeprops)
+            patch = patches.PathPatch(
+                path, lw=lw, edgecolor=color, zorder=1, **self.edgeprops
+            )
             self.ax.add_patch(patch)
 
     def draw(self):
         super(ArcPlot, self).draw()
-        xlimits = (-1, len(self.nodes) + 1)
+
+        left_limit = self.node_sizes[0]
+        right_limit = sum(r for r in self.node_sizes)
+        xlimits = (-left_limit, right_limit + 1)
         self.ax.set_xlim(*xlimits)
         self.ax.set_ylim(*xlimits)
+
+
+class GeoPlot(BasePlot):
+    """
+    Plotting object for GeoPlot.
+
+    User only has to specify the keyword arguments that specify the longitude
+    and latittude of a node.
+    """
+
+    def __init__(
+        self,
+        graph,
+        node_lat: str,
+        node_lon: str,
+        backend: str = "matplotlib",
+        **kwargs,
+    ):
+        """
+        Create the GeoPlot.
+
+        :param graph: The graph object.
+        :param node_lat: Node attribute that describes latitutde.
+        :param node_lon: Node attribute that describes longitude.
+        :param backend: Which backend to use - either `matplotlib` or `altair`.
+        """
+        self.graph = graph
+        self.node_lat = node_lat
+        self.node_lon = node_lon
+        # Set backend: matplotlib or altair
+        assert backend in ["matplotlib", "altair"]
+        self.backend = backend
+
+        # If the user chooses the altair backend, then we have to convert the
+        # graph data back into a nodelist and edgelist dataframes.
+        if self.backend == "altair":
+            self.node_df = to_pandas_nodes(self.graph)
+            self.edge_df = to_pandas_edges(
+                self.graph, x_kw=self.node_lon, y_kw=self.node_lat
+            )
+
+        super(GeoPlot, self).__init__(graph, **kwargs)
+
+    def compute_node_positions(self):
+        """
+        Extracts the node positions based on the specified longitude and
+        latitude keyword arguments.
+        """
+        xs = []
+        ys = []
+        self.locs = dict()
+
+        for node in self.nodes:
+            x = self.graph.nodes[node][self.node_lon]
+            y = self.graph.nodes[node][self.node_lat]
+            xs.append(x)
+            ys.append(y)
+            self.locs[node] = (x, y)
+
+        self.node_coords = {"x": xs, "y": ys}
+
+    def draw_nodes(self):
+        """
+        Draws nodes to the screen.
+
+        GeoPlot is the first plot kind to support an Altair backend in addition
+        to the usual matplotlib backend.
+        """
+        if self.backend == "matplotlib":
+            node_r = 0.005  # temporarily hardcoded.
+            for i, node in enumerate(self.nodes):
+                x = self.node_coords["x"][i]
+                y = self.node_coords["y"][i]
+                color = self.node_colors[i]
+                node_patch = patches.Ellipse(
+                    (x, y), node_r, node_r, lw=0, color=color, zorder=2
+                )
+                self.ax.add_patch(node_patch)
+        elif self.backend == "altair":
+            self.node_chart = (
+                alt.Chart(self.node_df)
+                .mark_point()
+                .encode(
+                    alt.X(f"{self.node_lon}:Q", scale=alt.Scale(zero=False)),
+                    alt.Y(f"{self.node_lat}:Q", scale=alt.Scale(zero=False)),
+                )
+            )
+
+    def draw_edges(self):
+        """
+        Draws edges to screen.
+        """
+        if self.backend == "matplotlib":
+            for i, (n1, n2) in enumerate(self.edges):
+                x1, y1 = self.locs[n1]
+                x2, y2 = self.locs[n2]
+                color = self.edge_colors[i]
+                line = Line2D(
+                    xdata=[x1, x2],
+                    ydata=[y1, y2],
+                    color=color,
+                    zorder=0,
+                    alpha=0.3,
+                )
+                self.ax.add_line(line)
+        elif self.backend == "altair":
+            marker_attrs = dict()
+            marker_attrs["color"] = "black"  # MAGICNUMBER
+            marker_attrs["strokeWidth"] = 1  # MAGICNUMBER
+            self.edge_chart = (
+                alt.Chart(self.edge_df)
+                .mark_line(**marker_attrs)
+                .encode(
+                    alt.X(f"{self.node_lon}:Q"),
+                    alt.Y(f"{self.node_lat}:Q"),
+                    detail="edge",
+                )
+            )
+
+    def draw(self):
+        if self.backend == "matplotlib":
+            super(GeoPlot, self).draw()
+        elif self.backend == "altair":
+            warn("altair backend still in development, please use matplotlib.")
+            # self.draw_nodes()
+            # self.draw_edges()
+            # self.viz = self.edge_chart + self.node_chart
+            # return self.viz.interactive()
